// Copyright (c) 2010  John Abbott
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
using namespace CoCoA;
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "This program illustrates use of the pseudo-random number generator of CoCoALib.\n"
  "The numbers are independent and uniformly distributed in the given range; both \n"
  "ends of the range are reachable.                                               \n"
  "See RandomBitStream if you want random bits,                                   \n"
  "& RandomLongStream if you want random machine integers.                        \n";

const string LongDescription =
  "CoCoALib offers a way to make uniform pseudo-random number generators.      \n"
  "When creating the generator you must specify the (inclusive) upper and lower\n"
  "bounds for the random numbers which will be generated.  When creating a     \n"
  "generator you may specify a `seed'; this allows different pseudo-random     \n"
  "sequences to be produced, though the sequence is completely determined by   \n"
  "its initial seed value.                                                     \n";
//----------------------------------------------------------------------

void program()
{
  GlobalManager CoCoAFoundations;

  GlobalOutput() << ShortDescription << endl;

  GlobalOutput() << "Here are 20 random samples from the uniform distribution on [-10^10,+10^10]\n";
  const ZZ N = power(10,10);
  RandomZZStream RZS(-N,N);
  for (int i=0; i < 20; ++i)
    GlobalOutput() << sample(RZS) << " ";
  GlobalOutput() << endl << endl;

  // If you prefer you can RZS as an (endless) input iterator
  GlobalOutput() << "Here are 20 more random samples; this time from uniform distr on [0,10^99]\n";
  RandomZZStream RZS2(0,power(10,99));
  for (int i=0; i < 20; ++i)
  {
    ++RZS2;
    GlobalOutput() << *RZS2 << "\n";
  }
  GlobalOutput() << endl;
}

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(err);
  }
  catch (const std::exception& exc)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    GlobalErrput() << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-RandomZZStream1.C,v 1.1 2010/06/29 15:17:02 abbott Exp $
// $Log: ex-RandomZZStream1.C,v $
// Revision 1.1  2010/06/29 15:17:02  abbott
// New class RandomZZStream
//
//
