#ifdef CoCoA_WITH_NORMALIZ
//   Copyright (c)  2010 Anna M. Bigatti, Christof Soeger

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


// Source code for Frobby integration

#include "CoCoA/TmpNormaliz.H"
#include "CoCoA/TmpPPVector.H"
#include "CoCoA/ZZ.H"
#include "CoCoA/convert.H"
#include "CoCoA/error.H"
#include "CoCoA/error.H"

#include "CoCoA/io.H"  // just for debugging
#include <iostream>  // just for debugging

#include "gmpxx.h"
//#include "libnormaliz.cpp"
using libnormaliz::Cone;
using libnormaliz::ConeProperties;



#include <list>
using std::list;
#include <vector>
using std::vector;


//template class libnormaliz::Cone<long>;
//template class libnormaliz::Cone<mpz_class>;


namespace CoCoA
{

namespace NormalizConeImpl
{

  NormalizCone NormalizConeLong(const std::vector<std::vector<ZZ> >& v, int InputType);
  NormalizCone NormalizConeMPZ(const std::vector<std::vector<ZZ> >& v, int InputType);

  class LongImpl: public NormalizConeBase
  {
  public:
    // waiting for Normaliz Team decisions on input
    LongImpl(const std::vector<std::vector<ZZ> >& v, int InputType);
    LongImpl(const PPVector& v, int InputType);
    friend NormalizCone NormalizConeLong(const std::vector<std::vector<ZZ> >& v, int InputType);
  private:
    void myNormalizComputation(const libnormaliz::ConeProperties& CPs) const;
    void myNormalizComputation(libnormaliz::ConeProperty::Enum CP) const;    
    void myHilbertBasis(std::vector<std::vector<ZZ> >& v) const;
    void myHt1Elements(std::vector<std::vector<ZZ> >& v) const;
    void mySupportHyperplanes(std::vector<std::vector<ZZ> >& v) const;
    void myTriangulation(std::vector<std::vector<ZZ> >& v) const;
  private:
    mutable libnormaliz::Cone<long> myConeLong; // be careful!
  };


  class MPZImpl: public NormalizConeBase
  {
  public:
    // waiting for Normaliz Team decisions on input
    MPZImpl(const std::vector<std::vector<ZZ> >& v, int InputType);
    MPZImpl(const PPVector& v, int InputType);
    friend NormalizCone NormalizConeMPZ(const std::vector<std::vector<ZZ> >& v, int InputType);
  private:
    void myNormalizComputation(const libnormaliz::ConeProperties& CPs) const;
    void myNormalizComputation(libnormaliz::ConeProperty::Enum CP) const;    
    void myHilbertBasis(std::vector<std::vector<ZZ> >& v) const;
    void myHt1Elements(std::vector<std::vector<ZZ> >& v) const;
    void mySupportHyperplanes(std::vector<std::vector<ZZ> >& v) const;
    void myTriangulation(std::vector<std::vector<ZZ> >& v) const;
  private:
    mutable libnormaliz::Cone<mpz_class> myConeMPZ; // be careful
  };
}
  
  

  void HilbertBasis(std::vector<std::vector<ZZ> >& v, const NormalizCone& c)
  { c->myHilbertBasis(v); }

  void Ht1Elements(std::vector<std::vector<ZZ> >& v, const NormalizCone& c)
  { c->myHt1Elements(v); }
  
  void SupportHyperplanes(std::vector<std::vector<ZZ> >& v, const NormalizCone& c)
  { c->mySupportHyperplanes(v); }
  
  void Triangulation(std::vector<std::vector<ZZ> >& v, const NormalizCone& c)
  { c->myTriangulation(v); }


  NormalizCone NewNormalizConeLong(const std::vector<std::vector<ZZ> >& v, int InputType)
  { return NormalizCone(new NormalizConeImpl::LongImpl(v, InputType)); }
  

  NormalizCone NewNormalizConeMPZ(const std::vector<std::vector<ZZ> >& v, int InputType)
  { return NormalizCone(new NormalizConeImpl::MPZImpl(v, InputType)); }



  namespace  // conversion functions
  {
    std::vector<ZZ> LongVToZZV(const std::vector<long>& VIn)
    {
      std::vector<ZZ> v;
      v.reserve(VIn.size());
      for (vector<long>::const_iterator it=VIn.begin(); it!=VIn.end(); ++it)
        v.push_back(ZZ(*it));
      return v;
    }


    std::vector<ZZ> MPZ_classVToZZV(const std::vector<mpz_class>& VIn)
    {
      std::vector<ZZ> v;
      v.reserve(VIn.size());
      for (vector<mpz_class>::const_iterator it=VIn.begin(); it!=VIn.end(); ++it)
        v.push_back(ZZ(CopyFromMPZ, it->get_mpz_t()));
      return v;
    }


    std::vector<long> ZZVToLongV(const std::vector<ZZ>& VIn)
    {
      std::vector<long> v;
      long n;
      v.reserve(VIn.size());
      for (vector<ZZ>::const_iterator it=VIn.begin(); it!=VIn.end(); ++it)
        if (IsConvertible(n, *it))  v.push_back(n);
        else CoCoA_ERROR(ERR::BadConvert, "normaliz: ZZVToLongV");
      return v;
    }


    std::vector<mpz_class> ZZVToMPZ_classV(const std::vector<ZZ>& VIn)
    {
      std::vector<mpz_class> v;
      v.reserve(VIn.size());
      for (vector<ZZ>::const_iterator it=VIn.begin(); it!=VIn.end(); ++it)
        v.push_back(mpz_class(mpzref(*it)));
      return v;
    }


    void ConvertFromNormaliz(std::vector<std::vector<ZZ> >& v, const std::list<std::vector<long> >& l)
    {
      v.clear();      // not exception safe
      //      transform(l.begin(), l.end(), v.begin(), LongVToZZV);
      for (list<vector<long> >::const_iterator it=l.begin(); it!=l.end(); ++it)
        v.push_back(LongVToZZV(*it));
    }    


    void ConvertFromNormaliz(std::vector<std::vector<ZZ> >& v, const std::list<std::vector<mpz_class> >& l)
    {
      v.clear();      // not exception safe
      //      transform(l.begin(), l.end(), v.begin(), LongVToZZV);
      for (list<vector<mpz_class> >::const_iterator it=l.begin(); it!=l.end(); ++it)
        v.push_back(MPZ_classVToZZV(*it));
    }    


    std::list<std::vector<long> >  ReturnListVecLong(const std::vector<std::vector<ZZ> >& v)
    {
      std::list<std::vector<long> > l;
      //      transform(v.begin(), v.end(), l.begin(), ZZVToLongV);
      for (vector<vector<ZZ> >::const_iterator it=v.begin(); it!=v.end(); ++it)
        l.push_back(ZZVToLongV(*it));
      //      std::clog << l << std::endl;
      return l;
    }


    std::list<std::vector<mpz_class> >  ReturnListVecMPZ_class(const std::vector<std::vector<ZZ> >& v)
    {
      std::list<std::vector<mpz_class> > l;
      //      transform(v.begin(), v.end(), l.begin(), ZZVToLongV);
      for (vector<vector<ZZ> >::const_iterator it=v.begin(); it!=v.end(); ++it)
        l.push_back(ZZVToMPZ_classV(*it));
      //      std::clog << l << std::endl;
      return l;
    }


  }


namespace NormalizConeImpl
{
  LongImpl::LongImpl(const std::vector<std::vector<ZZ> >& v, int InputType):
    myConeLong(ReturnListVecLong(v), InputType)
  {}
  
  
  MPZImpl::MPZImpl(const std::vector<std::vector<ZZ> >& v, int InputType):
    myConeMPZ(ReturnListVecMPZ_class(v), InputType)
  {}
  

  void LongImpl::myNormalizComputation(const libnormaliz::ConeProperties& CPs) const
  { myConeLong.compute(CPs); }


  void MPZImpl::myNormalizComputation(const libnormaliz::ConeProperties& CPs) const
  { myConeMPZ.compute(CPs); }


  void LongImpl::myNormalizComputation(libnormaliz::ConeProperty::Enum CP) const
  {
    if (!myConeLong.isComputed(CP))
      myConeLong.compute(ConeProperties(CP));
    if (!myConeLong.isComputed(CP))
      CoCoA_ERROR("not computed", "myNormalizComputation long");
  }


  void MPZImpl::myNormalizComputation(libnormaliz::ConeProperty::Enum CP) const
  {
    if (!myConeMPZ.isComputed(CP))
      myConeMPZ.compute(ConeProperties(CP));
    if (!myConeMPZ.isComputed(CP))
      CoCoA_ERROR("not computed", "myNormalizComputation MPZ");
  }


  

  void LongImpl::myHilbertBasis(std::vector<std::vector<ZZ> >& v) const
  {
    myNormalizComputation(libnormaliz::ConeProperty::HilbertBasis);
    ConvertFromNormaliz(v, myConeLong.getHilbertBasis());
  }


  void LongImpl::myHt1Elements(std::vector<std::vector<ZZ> >& v) const
  {
    myNormalizComputation(libnormaliz::ConeProperty::Ht1Elements);
    ConvertFromNormaliz(v, myConeLong.getHt1Elements());
  }


  void LongImpl::mySupportHyperplanes(std::vector<std::vector<ZZ> >& v) const
  {
    myNormalizComputation(libnormaliz::ConeProperty::SupportHyperplanes);
    ConvertFromNormaliz(v, myConeLong.getSupportHyperplanes());
  }


  void LongImpl::myTriangulation(std::vector<std::vector<ZZ> >& v) const
  {
    myNormalizComputation(libnormaliz::ConeProperty::Triangulation);
    ConvertFromNormaliz(v, myConeLong.getTriangulation());
  }


  void MPZImpl::myHilbertBasis(std::vector<std::vector<ZZ> >& v) const
  {
    myNormalizComputation(libnormaliz::ConeProperty::HilbertBasis);
    ConvertFromNormaliz(v, myConeMPZ.getHilbertBasis());
  }


  void MPZImpl::myHt1Elements(std::vector<std::vector<ZZ> >& v) const
  {
    myNormalizComputation(libnormaliz::ConeProperty::Ht1Elements);
    ConvertFromNormaliz(v, myConeMPZ.getHt1Elements());
  }


  void MPZImpl::mySupportHyperplanes(std::vector<std::vector<ZZ> >& v) const
  {
    myNormalizComputation(libnormaliz::ConeProperty::SupportHyperplanes);
    ConvertFromNormaliz(v, myConeMPZ.getSupportHyperplanes());
  }


  void MPZImpl::myTriangulation(std::vector<std::vector<ZZ> >& v) const
  {
    myNormalizComputation(libnormaliz::ConeProperty::Triangulation);
    ConvertFromNormaliz(v, myConeMPZ.getTriangulation());
  }


} // namespace Normaliz
} // namespace CoCoA

// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/TmpNormaliz.C,v 1.6 2010/10/12 11:22:54 bigatti Exp $
// $Log: TmpNormaliz.C,v $
// Revision 1.6  2010/10/12 11:22:54  bigatti
// -- TmpNormaliz.H simplified:
//    now NormalizCone is a smart pointer to NormalizConeBase
//    and the concrete classes are entirely in the .C file
// -- added Ht1Elements, SupportHyperplanes, Triangulation
// -- added some text in ex-Normaliz1 and 2
//
// Revision 1.5  2010/10/08 13:40:37  bigatti
// -- moved inclusion of libnormaliz.cpp and instantiation of types
//    into TmpNormalizTypes.C
//
// Revision 1.4  2010/10/08 10:39:32  bigatti
// -- extended interface for normaliz
//

#endif
